@@ -1,9 +1,16 @@
-import types
 import re
-import time
-
-import threading
 from socket import AF_INET, SOCK_DGRAM, socket
+import threading
+import time
+import types
+
+try:
+    from setproctitle import setproctitle
+except ImportError:
+    setproctitle = None
+
+from daemon import Daemon
+
 
 __all__ = ['Server']
 
@@ -14,7 +21,6 @@ def _clean_key(k):
         k.replace('/','-').replace(' ','_')
     )
 
-
 TIMER_MSG = '''stats.timers.%(key)s.lower %(min)s %(ts)s
 stats.timers.%(key)s.count %(count)s %(ts)s
 stats.timers.%(key)s.mean %(mean)s %(ts)s
@@ -23,7 +29,7 @@ stats.timers.%(key)s.upper_%(pct_threshold)s %(max_threshold)s %(ts)s
 '''
 
 class Server(object):
-
+    
     def __init__(self, pct_threshold=90, debug=False, graphite_host='localhost', graphite_port=2003):
         self.buf = 1024
         self.flush_interval = 10000
@@ -36,7 +42,6 @@ class Server(object):
         self.timers = {}
         self.flusher = 0
 
-    
     def process(self, data):
         key, val = data.split(':')
         key = _clean_key(key)
@@ -47,19 +52,18 @@ class Server(object):
         if (fields[1] == 'ms'):
             if key not in self.timers:
                 self.timers[key] = []
-            self.timers[key].append(int(fields[0] or 0))
+            self.timers[key].append(float(fields[0] or 0))
         else:
             if len(fields) == 3:
                 sample_rate = float(re.match('^@([\d\.]+)', fields[2]).groups()[0])
             if key not in self.counters:
                 self.counters[key] = 0;
-            self.counters[key] += int(fields[0] or 1) * (1 / sample_rate)
-        
+            self.counters[key] += float(fields[0] or 1) * (1 / sample_rate)
+
     def flush(self):
         ts = int(time.time())
         stats = 0
         stat_string = ''
-        self.pct_threshold = 10
         for k, v in self.counters.items():
             v = float(v) / (self.flush_interval / 1000)
             msg = 'stats.%s %s %s\n' % (k, v, ts)
@@ -79,7 +83,7 @@ class Server(object):
                 max_threshold = max
 
                 if count > 1:
-                    thresh_index = int(((100.0 - self.pct_threshold) / 100) * count)
+                    thresh_index = int((self.pct_threshold / 100.0) * count)
                     max_threshold = v[thresh_index - 1]
                     total = sum(v[:thresh_index-1])
                     mean = total / thresh_index
@@ -97,7 +101,7 @@ class Server(object):
                     'ts': ts,
                 }
                 stats += 1
-        
+
         stat_string += 'statsd.numStats %s %d' % (stats, ts)
 
         graphite = socket()
@@ -127,7 +131,7 @@ class Server(object):
         def signal_handler(signal, frame):
                 self.stop()
         signal.signal(signal.SIGINT, signal_handler)
-        
+
         self._set_timer()
         while 1:
             data, addr = self._sock.recvfrom(self.buf)
@@ -136,9 +140,17 @@ class Server(object):
     def stop(self):
         self._timer.cancel()
         self._sock.close()
-        
 
-if __name__ == '__main__':
+
+class ServerDaemon(Daemon):
+    def run(self, options):
+        if setproctitle:
+            setproctitle('pystatsd')
+        server = Server(pct_threshold=options.pct, debug=options.debug)
+        server.serve(options.name, options.port, options.graphite_host,
+                     options.graphite_port)
+
+def run_server():
     import sys
     import argparse
     parser = argparse.ArgumentParser()
@@ -148,6 +160,21 @@ if __name__ == '__main__':
     parser.add_argument('--graphite-port', dest='graphite_port', help='port to connect to graphite on', type=int, default=2003)
     parser.add_argument('--graphite-host', dest='graphite_host', help='host to connect to graphite on', type=str, default='localhost')
     parser.add_argument('-t', '--pct', dest='pct', help='stats pct threshold', type=int, default=90)
+    parser.add_argument('-D', '--daemon', dest='daemonize', action='store_true', help='daemonize', default=False)
+    parser.add_argument('--pidfile', dest='pidfile', action='store', help='pid file', default='/tmp/pystatsd.pid')
+    parser.add_argument('--restart', dest='restart', action='store_true', help='restart a running daemon', default=False)
+    parser.add_argument('--stop', dest='stop', action='store_true', help='stop a running daemon', default=False)
     options = parser.parse_args(sys.argv[1:])
 
-    Server(pct_threshold=options.pct, debug=options.debug).serve(options.name, options.port, options.graphite_host, options.graphite_port)
+    daemon = ServerDaemon(options.pidfile)
+    if options.daemonize:
+        daemon.start(options)
+    elif options.restart:
+        daemon.restart(options)
+    elif options.stop:
+        daemon.stop()
+    else:
+        daemon.run(options)
+
+if __name__ == '__main__':
+    run_server()
\ No newline at end of file
